# Temperature units
TEMP_CELSIUS = "°C"

# Supported features
SUPPORT_TARGET_TEMPERATURE = 1
SUPPORT_PRESET_MODE = 16

# HVAC modes
HVAC_MODE_OFF = "off"
HVAC_MODE_HEAT = "heat"

# HVAC states
CURRENT_HVAC_OFF = "off"
CURRENT_HVAC_HEAT = "heating"
CURRENT_HVAC_IDLE = "idle"

# Supported presets
PRESET_FOLLOW_SCHEDULE = "Follow Schedule"
PRESET_PERMANENT_HOLD = "Permanent Hold"
PRESET_OFF = "Off"